import {
    Box,
    Table,
    TableBody,
    TableCell,
    TableHead,
    TableRow,
    Typography,
    Skeleton,
} from "@mui/material";
import { styled } from "@mui/material/styles";
import { useState, useMemo } from "react";

import useMediaQuery from "@mui/material/useMediaQuery";

import { formatBalance } from "./StakeBalances";
import { useAccount, useChainId } from "wagmi";

import DataRow from "../../../components/DataRow/DataRow";
import InfoTooltip from "../../../components/Tooltip/InfoTooltip";
import Paper from "../../../components/Paper/Paper";
import Token from "../../../components/Token/Token";
import { PrimaryButton, SecondaryButton } from "../../../components/Button";
import { Tab, Tabs } from "../../../components/Tabs/Tabs";

import { DecimalBigNumber } from "../../../helpers/DecimalBigNumber";
import { prettifySecondsInDays } from "../../../helpers/timeUtil";
import { formatNumber } from "../../../helpers";
import { STAKING_ADDRESSES } from "../../../constants/addresses";
import { useCurrentIndex, useWarmupInfo } from "../../../hooks/staking";
import { useBalanceForShares } from "../../../hooks/tokens";

import ClaimConfirmationModal from "./ClaimConfirmationModal";

const PREFIX = "ExternalStakePools";

const classes = {
    stakePoolsWrapper: `${PREFIX}-stakePoolsWrapper`,
    stakePoolHeaderText: `${PREFIX}-stakePoolHeaderText`,
    poolPair: `${PREFIX}-poolPair`,
    poolName: `${PREFIX}-poolName`,
};

const StyledTableHeader = styled(TableHead)(({ theme }) => ({
    [`&.${classes.stakePoolHeaderText}`]: {
        color: theme.palette.text.secondary,
        lineHeight: 1.4,
    },
}));

export const ClaimsArea = ({ chainId, address, epoch }) => {
    const isSmallScreen = useMediaQuery("(max-width: 745px)");

    const [confirmationModalOpen, setConfirmationModalOpen] = useState(false);
    const [isPayoutGhst, setIsPayoutGhst] = useState(localStorage.getItem("stake-isGhstPayout")
        ? localStorage.getItem("stake-isGhstPayout")
        : false
    );

    const { warmupInfo: claim, refetch: claimRefetch } = useWarmupInfo(chainId, address);
    const { currentIndex, refetch: currentIndexRefetch } = useCurrentIndex(chainId);
    const { balanceForShares } = useBalanceForShares(chainId, "STNK", claim.shares);

    const closeConfirmationModal = () => {
        setConfirmationModalOpen(false);
        claimRefetch();
        currentIndexRefetch();
    }

    if (claim.shares === 0n) return <></>;

    const setIsPayoutGhstInner = (value) => {
        setIsPayoutGhst(value);
        localStorage.setitem("bond-isGhstPayout", value);
    }

    const warmupTooltip = `Your claim earns rebases during warmup. You can emergency withdraw, but this forfeits the rebases`;

    return (
        <>
            <ClaimConfirmationModal
                open={confirmationModalOpen}
                onClose={() => closeConfirmationModal()}
                chainid={chainId}
                receiver={address}
                receiveAmount={claim.expiry > epoch.number ? claim.deposit : isPayoutGhst ? balanceForShares.div(currentIndex) : balanceForShares}
                outputToken={claim.expiry > epoch.number ? "FTSO" : isPayoutGhst ? "GHST" : "STNK"}
                action={claim.expiry > epoch.number ? "forfeit" : "claim"}
                chainId={chainId}
            />
                <Paper
                    fullWidth
                    headerContent={
                        <Box
                            width="100%"
                            display="flex"
                            alignItems={isSmallScreen ? "start" : "center"}
                            justifyContent="space-between"
                            flexDirection={isSmallScreen ? "column" : "row"}
                        >
                            <Typography variant="h6">Your active {isPayoutGhst ? "GHST" : "STNK"} claim</Typography>
                            <Tabs
                                centered
                                textColor="primary"
                                indicatorColor="primary"
                                value={isPayoutGhst ? 1 : 0}
                                aria-label="Claim token tabs"
                                onChange={(_, view) => setIsPayoutGhstInner(view === 1)}
                                TabIndicatorProps={{ style: { display: "none" } }}
                            >
                                <Tab aria-label="payout-stnk-button" label="STNK" style={{ fontSize: "1rem" }} />
                                <Tab aria-label="payout-ghst-button" label="GHST" style={{ fontSize: "1rem" }} />
                            </Tabs>
                        </Box>
                    }
                    tooltip={warmupTooltip}
                    style={{ margin: "0", padding: "20px 0" }}
                >
                    {isSmallScreen ? (
                        <MobileClaimInfo
                            setConfirmationModalOpen={setConfirmationModalOpen}
                            prepareBalance={isPayoutGhst ? balanceForShares.div(currentIndex) : balanceForShares}
                            isPayoutGhst={isPayoutGhst}
                            claim={claim}
                            epoch={epoch}
                            isClaimable={claim.expiry > epoch.number}
                        />
                    ) : (
                        <Table>
                            <StyledTableHeader className={classes.stakePoolHeaderText}>
                                <TableRow>
                                    <TableCell style={{ width: "200px", padding: "8px 0" }}>Asset</TableCell>
                                    <TableCell style={{ width: "200px", padding: "8px 0" }}>Amount</TableCell>
                                    <TableCell style={{ width: "150px", padding: "8px 0" }}>Claimable In</TableCell>
                                    <TableCell></TableCell>
                                </TableRow>
                            </StyledTableHeader>
                            <ClaimInfo
                                setConfirmationModalOpen={setConfirmationModalOpen}
                                prepareBalance={isPayoutGhst ? balanceForShares.div(currentIndex) : balanceForShares}
                                isPayoutGhst={isPayoutGhst}
                                claim={claim}
                                epoch={epoch}
                                isClaimable={claim.expiry > epoch.number}
                            />
                        </Table>

                    )}
                </Paper>
        </>
    );
};

const ClaimInfo = ({ setConfirmationModalOpen, prepareBalance, claim, epoch, isClaimable, isPayoutGhst }) => {
    return (
        <TableBody>
            <TableRow>
                <TableCell style={{ padding: "8px 8px 8px 0" }}>
                    <Box display="flex" flexDirection="row" alignItems="center" style={{ whiteSpace: "nowrap" }}>
                        <Token key={isPayoutGhst ? "GHST" : "STNK"} name={isPayoutGhst ? "GHST" : "STNK"} />
                        <Box marginLeft="14px" marginRight="10px">
                            <Typography>{isPayoutGhst ? "GHST" : "STNK"}</Typography>
                        </Box>
                    </Box>
                </TableCell>
                <TableCell style={{ padding: "8px 8px 8px 0" }}>
                    <Typography gutterBottom={false} style={{ lineHeight: 1.4 }}>
                        {`${formatNumber(prepareBalance, 5)} ${isPayoutGhst ? "GHST" : "STNK"}`}
                    </Typography>
                </TableCell>
                <TableCell style={{ padding: "8px 8px 8px 0" }}>
                    {prettifySecondsInDays(epoch.length * (claim.expiry - epoch.number))}
                </TableCell>
                <TableCell style={{ padding: "8px 0" }}>
                    <ActionButtons
                        setConfirmationModalOpen={setConfirmationModalOpen}
                        isClaimable={isClaimable}
                    />
                </TableCell>
            </TableRow>
        </TableBody>
    );
};

const MobileClaimInfo = ({ setConfirmationModalOpen, prepareBalance, epoch, claim, isPayoutGhst, isClaimable }) => {
    return (
        <Box mt="10px">
            <Box display="flex" flexDirection="row" alignItems="center" style={{ whiteSpace: "nowrap" }}>
                <Token key={isPayoutGhst ? "GHST" : "STNK"} name={isPayoutGhst ? "GHST" : "STNK"} />
                <Box marginLeft="14px" marginRight="10px">
                    <Typography>{isPayoutGhst ? "GHST" : "STNK"}</Typography>
                </Box>
            </Box>

            <DataRow
                title="Amount"
                balance={formatNumber(prepareBalance, 7)}
            />

            <DataRow
                title="Claimed in"
                balance={prettifySecondsInDays(epoch.length * (claim.expiry - epoch.number))}
            />

            <ActionButtons
                isSmallScreen={true}
                setConfirmationModalOpen={setConfirmationModalOpen}
                isClaimable={isClaimable}
            />
        </Box>
    );
};

const ActionButtons = ({ setConfirmationModalOpen, isSmallScreen = false, isClaimable = false }) => {
    return (
        <Box
            display="flex"
            flexDirection={isSmallScreen ? "column" : "row"}
            alignItems="center"
            justifyContent="center"
        >
            <SecondaryButton
                loading={false}
                fullWidth={isSmallScreen}
                sx={{ flexGrow: 1 }}
                onClick={() => setConfirmationModalOpen(true)}
                disabled={!isClaimable}
            >
                Emergency Withdrawal
            </SecondaryButton>
            <PrimaryButton
                fullWidth={isSmallScreen}
                loading={false}
                sx={{ flexGrow: 1 }}
                onClick={() => setConfirmationModalOpen(true)}
                disabled={isClaimable}
            >
                Claim
            </PrimaryButton>
        </Box>
    );
};
