import Metric from "../../../components/Metric/Metric";
import { formatCurrency, formatNumber } from "../../../helpers";

import { useReadContract } from "wagmi";
import { DecimalBigNumber }from "../../../helpers/DecimalBigNumber";

import { useCurrentIndex } from "../../../hooks/staking";
import { useTotalReserves } from "../../../hooks/treasury";
import { useFtsoPrice } from "../../../hooks/prices";
import { useTotalSupply, useCirculatingSupply } from "../../../hooks/tokens";

export const CurrentIndex = props => {
    const { currentIndex } = useCurrentIndex(props.chainId);

    const _props = {
        ...props,
        label: `Current Index`,
        tooltip: `The current index indicates the amount of FTSO a holder would possess if they had staked and maintained 1 FTSO since its launch.`,
    };

    if (currentIndex) _props.metric = `${formatNumber(currentIndex, 2)} FTSO`;
    else _props.isLoading = true;

    return <Metric {..._props} />;
};

export const GhostPrice = props => {
    const ftsoPrice = useFtsoPrice(props.chainId);
    const { currentIndex } = useCurrentIndex(props.chainId);
    const ghstPrice = ftsoPrice * currentIndex;

    const _props = {
        ...props,
        label: "GHST " + `Price`,
        tooltip: "1 GHST = 1 FTSO x Current Index",
    };

    if (ghstPrice) _props.metric = formatCurrency(ghstPrice, 2);
    else _props.isLoading = true;

    return <Metric {..._props} />;
};

export const FatsoPrice = props => {
    const ftsoPrice = useFtsoPrice(props.chainId);

    const _props = {
        ...props,
        label: "FTSO " + `Price`,
        tooltip: `Weighted FTSO Price Across V2 DEXs on the chosen EVM Chain`,
    };

    if (ftsoPrice) _props.metric = formatCurrency(ftsoPrice, 2);
    else _props.isLoading = true;

    return <Metric {..._props} />;
};

export const CirculatingSupply = props => {
    const { totalSupply } = useTotalSupply(props.chainId, "FTSO");
    const circulatingSupply = useCirculatingSupply(props.chainId, "STNK");

    const _props = {
        ...props,
        label: `Circulating / Total`,
        tooltip: `Circulating supply refers to the amount of STNK in circulation, excluding those held by the protocol in its treasury. However, STNK allocated to Protocol-Owned Liquidity is considered part of the circulating supply.`,
    };

    if (circulatingSupply && totalSupply) _props.metric = `${formatNumber(circulatingSupply, 0)}/${formatNumber(totalSupply, 0)}`;
    else _props.isLoading = true;

    return <Metric {..._props} />;
};

export const FatsoBacking = props => {
    const { totalSupply } = useTotalSupply(props.chainId, "FTSO");
    const totalReserves = useTotalReserves(props.chainId);

    const backing = totalSupply.eq(new DecimalBigNumber(0, 0)) ?
        new DecimalBigNumber(0n, 0)
        :
        totalReserves.div(totalSupply)

    const _props = {
        ...props,
        label: `Backing per FTSO`,
        tooltip: `The total amount of stablecoins held by the ghostDAO treasury to support the value of each FTSO in circulation.`
    };

    if (backing) _props.metric = formatCurrency(backing, 2);
    else _props.isLoading = true;

    return <Metric {..._props} />;
};

export const TreasuryMarketCap = props => {
    const ftsoPrice = useFtsoPrice(props.chainId);
    const { totalSupply } = useTotalSupply(props.chainId, "FTSO");
    const marketCap = ftsoPrice.mul(totalSupply);

    const _props = {
        ...props,
        label: `Market Cap`,
        tooltip: `Market Cap = FTSO Price × FTSO Total Supply`
    };

    if (marketCap) _props.metric = formatCurrency(marketCap, 2);
    else _props.isLoading = true;

    return <Metric {..._props} />;
};
