import { useReadContract, useToken, useBalance as useInnerBalance } from "wagmi";
import { simulateContract, writeContract, waitForTransactionReceipt } from "@wagmi/core";
import toast from "react-hot-toast";

import { getTokenAbi, getTokenAddress, getTokenDecimals } from "../helpers";

import { DecimalBigNumber } from "../../helpers/DecimalBigNumber";
import { shorten } from "../../helpers";
import { config } from "../../config";

export const useTotalSupply = (chainId, name) => {
    const contractAddress = getTokenAddress(chainId, name);
    const { data, refetch } = useToken({
        address: contractAddress,
        scopeKey: `token-${contractAddress}-${chainId}`,
        chainId
    });

    const totalSupplyPrepared = data ? data.totalSupply.value : 0n;
    const decimals = data ? data.decimals : 0;
    const totalSupply = new DecimalBigNumber(totalSupplyPrepared, decimals);

    return { totalSupply, refetch };
};

export const useBalance = (chainId, name, address) => {
    const contractAddress = getTokenAddress(chainId, name);
    const { data, refetch } = useInnerBalance({
        address,
        chainId,
        scopeKey: `balance-${contractAddress}-${address}-${chainId}`,
        token: contractAddress,
    });

    const balancePrepared = data ? data.value : 0n;
    const decimals = data ? data.decimals : getTokenDecimals(name);

    const balance = new DecimalBigNumber(balancePrepared, decimals);

    return { balance, refetch, contractAddress };
}

export const useAllowance = (chainId, name, owner, spender, decimals) => {
    const contractAddress = getTokenAddress(chainId, name);
    const { data, refetch } = useReadContract({
        abi: getTokenAbi(name),
        address: contractAddress,
        functionName: "allowance",
        args: [owner, spender],
        scopeKey: `allowance-${contractAddress}-${owner}-${spender}-${chainId}`,
        chainId: chainId,
    });

    const allowancePrepared = data ? data : 0n;
    decimals = decimals ? decimals : getTokenDecimals(name);
    const allowance = new DecimalBigNumber(allowancePrepared, decimals);

    return { allowance, refetch };
}

export const useTokenSymbol = (chainId, name) => {
    const contractAddress = getTokenAddress(chainId, name);
    const { data, refetch } = useReadContract({
        abi: getTokenAbi(name),
        address: contractAddress,
        functionName: "symbol",
        scopeKey: `symbol-${contractAddress}-${chainId}`,
        chainId: chainId,
    });

    const symbol = data ? data : "";
    return { symbol, refetch };
}

export const useConversionRate = (chainId, name) => {
    const contractAddress = getTokenAddress(chainId, name);
    const { data: rateRaw } = useReadContract({
        abi: getTokenAbi(name),
        address: contractAddress,
        functionName: "conversionRate",
        scopeKey: `conversionRate-${contractAddress}-${chainId}`,
        chainId: chainId,
    });

    const ratePrepared = rateRaw ? rateRaw : 0n;
    const rate = new DecimalBigNumber(ratePrepared, 0);

    return rate;
}

export const useCirculatingSupply = (chainId, name) => {
    const contractAddress = getTokenAddress(chainId, name);
    const { data: circulatingSupplyRaw } = useReadContract({
        abi: getTokenAbi(name),
        address: contractAddress,
        functionName: "circulatingSupply",
        scopeKey: `circulatingSupply-${contractAddress}-${chainId}`,
        chainId: chainId,
    });

    const circulatingSupplyPrepared = circulatingSupplyRaw ? circulatingSupplyRaw : 0n;
    const circulatingSupply = new DecimalBigNumber(circulatingSupplyPrepared, 9);

    return circulatingSupply;
};

export const useBalanceForShares = (chainId, name, amount) => {
    const contractAddress = getTokenAddress(chainId, name);
    const { data: balanceForSharesRaw } = useReadContract({
        abi: getTokenAbi(name),
        address: contractAddress,
        functionName: "balanceForShares",
        args: [amount],
        scopeKey: `balanceForShares-${chainId}`,
        chainId: chainId,
    });

    const balanceForSharesPrepared = balanceForSharesRaw ? balanceForSharesRaw : 0n;
    const balanceForShares = new DecimalBigNumber(balanceForSharesPrepared, 9);

    return { balanceForShares };
};

export const approveTokens = async (chainId, name, owner, spender, value) => {
    try {
        const { request } = await simulateContract(config, {
            abi: getTokenAbi(name),
            address: getTokenAddress(chainId, name),
            functionName: 'approve',
            args: [spender, value],
            account: owner,
            chainId: chainId
        });

        const txHash = await writeContract(config, request);
        await waitForTransactionReceipt(config, {
            hash: txHash,
            onReplaced: () => toast("Approve transaction was replaced. Wait for inclusion please."),
            chainId
        });

        toast.success(name + " tokens successfully approved to " + shorten(spender));
    } catch (err) {
        console.error(err);
        toast.error(name + " tokens approval failed. Check logs for error detalization.")
    }
}

export const mintDai = async (chainId, account, value) => {
    try {
        const { request } = await simulateContract(config, {
            abi: getTokenAbi("GDAI"),
            address: getTokenAddress(chainId, "GDAI"),
            functionName: 'mint',
            args: [account],
            account: account,
            value: value,
            chainId: chainId
        });

        const txHash = await writeContract(config, request);
        await waitForTransactionReceipt(config, {
            hash: txHash,
            onReplaced: () => toast("Mint transaction was replaced. Wait for inclusion please."),
            chainId
        });

        toast.success("gDAI successfully minted to your wallet! Funds will be used on Ghost Faucet.");
    } catch (err) {
        console.error(err);
        toast.error("Minting gDAI from the faucet failed. Check logs for error detalization.")
    }
}
